/* filename:    dsgepm.g
** description: The program maximizes the posterior density of the 
**              dsge model
** created:     02/06/02
*/

new;
library pgraph, user;
cls;
outwidth 256; 

format /mat /on /mb1 /ros 16,8;


/* Global defaults
*/
  __output = 1;
  _fcmptol = 1E-10;
  _graph = 1; 
  _verbose =2; 

#include dsgemod.src;
#include dsgelh.src;
#include pathspec.g
#include dsgespec.g

/******************************************************** 
** Import data
** series (nobs,2)
*/
#include loaddata.g

nobs  = rows(series);                     /* number of observations  */
YY    = series[1:nobs,.];   

/******************************************************** 
** Import the information for the prior density
*/
priorfile = lmodel $+ lprior $+ dataselstr $+ subTstr $+ "pri.out";
load path = ^priopath prior[npara+1,3] = ^priorfile;

pmean  = prior[1:npara,1];
pstdd  = prior[1:npara,2];
pshape = prior[1:npara,3];
pshape = pshape.*para_maskinv[.,mspec+1];

/******************************************************** 
** Starting Values for Maximization
*/

alp  = 0.64;    /* labor share */
bet  = 0.97;      /* discount factor */
gam  = 0.0033;  /* technical progress */
del  = 0.03;   /* depreciation */
nu   = 1.7;     /*preference parameter */ 
A0   =  2;     /*technological shock */  
B0   =  4;      /*preference shock */ 
rhoB = 0.96;    /* Persistence: preference shock */
sigA = 0.1;
sigB = 0.1; 
phi  = 30; 

if (mspec==0) or (mspec==1); 

   para  = alp | bet |gam | del |
           nu | A0 | B0 | rhoB |
           sigA | sigB ; 
   para_names = { alp bet gam del nu A0 B0 rhoB sigA sigB };

else;
   para  = alp | bet |gam | del |
           nu | A0 | B0 | rhoB |
           sigA | sigB | phi ; 
   para_names = { alp bet gam del nu A0 B0 rhoB sigA sigB phi };

endif;

para  = para.*para_maskinv[.,mspec+1] + para_fix[.,mspec+1].*para_mask[.,mspec+1];

npara = rows(para);

save path = ^priopath para_names;

/* Load transformation scheme for parameters
*/
if (mspec==0) or (mspec==1); 
transfile = "trans.out";
load path=^priopath _trspec[npara,4] = ^transfile;
_trspec[.,1] = _trspec[.,1].*para_maskinv[.,mspec+1];
else;
transfile = "trans2.out";
load path=^priopath _trspec[npara,4] = ^transfile;
_trspec[.,1] = _trspec[.,1].*para_maskinv[.,mspec+1];
endif;

/******************************************************** 
**      Calculate posterior density at starting values
**
*/
cls;
"Prior Density at Starting Value:" priodsge(para,pmean,pstdd,pshape);
"Posterior at Starting Value: " -objfcn(invtrans(para));
"Press key to continue";
w = keyw;
cls;

/*********************************************************
**       Maximize the posterior density
**
*/
cc = 0;
x0 = invtrans(para) + cc*( para_maskinv[.,mspec+1].*rndn(npara,1) );

H0= eye(npara)*1E-4;
nit = 4000;
crit= 1E-7;
/* Use personal optimization procedure
*/
{fh,xh,g,H,itct,fcount,retcode} = csminwel(x0,H0,crit,nit,&objfcn);

/********************************************************
** Save parameter estimates
*/
paraest = trans(xh);
paraest = real( paraest.*para_maskinv[.,mspec+1] + para_fix[.,mspec+1].*para_mask[.,mspec+1] );

oparaest         = postpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "pm";
create fhparaest = ^oparaest with PARAEST, 1, 8;
wr               = writer(fhparaest,paraest);
closeall fhparaest;   


/********************************************************
**       Estimation Output
**
*/
{lnpy, retcode, obsmean, obsvar} = evaldsge( paraest,mspec,YY);
lnprio  = priodsge( paraest, pmean, pstdd, pshape);

/* Compute gradient in terms of model parameters
*/
gradmod = g./ diag(gradp(&invtrans,paraest));

cls;
"Model              " mspec;
"Dataset            " datasel;
"Subsample          " SubT;
"===================================";
"Posterior Mode     " (lnpy+lnprio);
"Likelihood at Mode " lnpy;
"Prior at Mode      " lnprio;
"Number of Iteration "; itct; 
"Parameter   | Estimate |  Start-V  | Prior Mean | Prior Stdd | Gradient";

outmat = para_names'~paraest~trans(x0)~pmean~pstdd~gradmod;

let mask[1,6]=0 1 1 1 1 1;

let fmt[6,3]="-*.*s"  8 4
             "*.*lf"  8 4
             "*.*lf"  8 4
             "*.*lf"  8 4
             "*.*lf"  8 4
             "*.*lf"  8 4;

call printfm(outmat,mask,fmt);

if lnpy == -1E6 ;
   end;
endif;

/*********************************************************
**       Compute Residuals
**
*/

/* Moments */

resid1 = YY -obsmean;
resid1_m1 = meanc(resid1)';
resid1_m2 = meanc( (resid1 - resid1_m1)^2 )';
resid1_m3 = meanc( (resid1 - resid1_m1)^3 )';
resid1_m4 = meanc( (resid1 - resid1_m1)^4 )';

resid2 = (YY -obsmean)./sqrt(obsvar);
resid2_m1 = meanc(resid1)';
resid2_m2 = meanc( (resid2 - resid2_m1)^2 )';
resid2_m3 = meanc( (resid2 - resid2_m1)^3 )';
resid2_m4 = meanc( (resid2 - resid2_m1)^4 )';

/* Autocorrelation */

resid1_ar = zeros(1,2);
resid2_ar = zeros(1,2);

ind = 1;
do until ind > 2;
   resid1_ar[1,ind]=inv(resid1[1:nobs-1,ind]'*resid1[1:nobs-1,ind])
                    *resid1[1:nobs-1,ind]'*resid1[2:nobs,ind];
   resid2_ar[1,ind]=inv(resid2[1:nobs-1,ind]'*resid2[1:nobs-1,ind])
                    *resid2[1:nobs-1,ind]'*resid2[2:nobs,ind];
   ind = ind+1;
endo;


"Raw Residuals";
"-------------";
"Mean:" resid1_m1;
"Stdd:" sqrt(resid1_m2);
"Skew:" resid1_m3./resid1_m2^(3/2);
"Kurt:" resid1_m4./(3*resid1_m2^2);
"Corr:" resid1_ar;
"";
"Standardized Residuals";
"----------------------";
"Mean:" resid2_m1;
"Stdd:" sqrt(resid2_m2);
"Skew:" resid2_m3./resid2_m2^(3/2);
"Kurt:" resid2_m4./(3*resid2_m2^2);
"Corr:" resid2_ar;
"";
ti=seqa(1959.25,0.25,nobs); 
titlestr = "Output" $|
             "Hour"; 

ystr     = "Raw Residual " $|
           "Stdd Residual"; 

graphset;
fonts("microb");
_ptitlht = 0.4;
_paxht   = 0.4;
_pnumht  = 0.4;
_ptek    = resupath $+ "\\pmresid.tkf";
          
begwind;
window(2,5,0);


ser_ind = 1;
do until ser_ind > 2;

   ylabel(ystr[1]);
   title(titlestr[ser_ind]);
   xy(ti,resid1[.,ser_ind]);
 
   ser_ind = ser_ind+1;
   nextwind;

endo;

ser_ind = 1;
do until ser_ind > 2;

   ylabel(ystr[2]);
   title(titlestr[ser_ind]);
   xy(ti,resid2[.,ser_ind]);
 
   ser_ind = ser_ind+1;
   nextwind;

endo;

endwind;
end;


/****************************************************
                 PROCEDURES                       
***************************************************/

proc (1) = objfcn(para);
local lnpY, lnprio, obsmean, obsvar, modelpara, retcode;

/* likelihood 
*/
modelpara = trans(para);
modelpara = modelpara.*para_maskinv[.,mspec+1] + para_fix[.,mspec+1].*para_mask[.,mspec+1];

{lnpY,retcode,obsmean,obsvar} = evaldsge(modelpara,mspec,YY);

/* Evaluate the Prior distribution
*/
lnprio = priodsge(modelpara, pmean, pstdd, pshape);

/*
locate 25,1;
"Likelihood" lnpY  "Return Code" retcode;
*/

retp(real(-lnpY-lnprio));  /* We minimize the inverse of the likelihood fcn */
endp;

